use Test::More tests => 8;
use strict;

my $h;

BEGIN { use_ok('DBIx::ProcedureCall::ODBC') };
END {
    if ($h) {
        $h->{PrintError} = 0;
        eval {$h->do(q/drop procedure drop_me_1/)};
        eval {$h->do(q/drop procedure drop_me_2/)};
        eval {$h->do(q/drop procedure drop_me_3/)};
    }
};

eval {
	require DBI;
} or skip "could not load DBI module: $@", 8;

sub create_procedures
{
    my $h = shift;

    $h->do(<<'EOT');
create procedure drop_me_1 @ival int as
return @ival
EOT

    $h->do(<<'EOT');
create procedure drop_me_2 (@ival int, @result int output) as
begin
set @result = @ival + 1;
return @ival + 2;
end
EOT
    $h->do(<<'EOT');
create procedure drop_me_3 @ival int output as
set @ival = 1;
EOT
}

eval {
     $h = DBI->connect();
};

SKIP: {
    skip "Cannot connect to database - did you set DBI_USER, DBI_USER and DBI_PASS", 7 if !$h;

    my $driver = $h->{Driver}->{Name};

    skip "Not DBD::ODBC", 7 if $driver ne 'ODBC';

    my $dbname = $h->get_info(17); # DBI::SQL_DBMS_NAME
    skip "Microsoft SQL Server tests not supported using $dbname", 6
        unless ($dbname =~ /Microsoft SQL Server/i);

    create_procedures($h);

    $h->{PrintError} = 0;
    $h->{RaiseError} = 1;

    {
    	package T1;
    	eval q{
    		use DBIx::ProcedureCall qw(
    				drop_me_1
                    drop_me_2
                    drop_me_3:procedure
    				);};
    }

    my $result;
    ok ( $result = T1::drop_me_1($h, 1), 'simple call to function');
    is($result, 1, 'simple return value');

    my $out;
    ok ( $result = T1::drop_me_2($h, 1, \$out), 'function with out param');
    is($out, 2, 'out function result');
    is($result, 3, 'out param on function return value');

    ok ( $result = T1::drop_me_3($h, \$out), 'procedure with out param ');
    is($result, 1, 'out param');
};

